function DemoPlasmonicMetasurfaces()
clc;
clear all;
close all;

%% parameters of nanoparticles
n=1000;
wavelength=linspace(150*10^(-9),350*10^(-9),n);
eps_c = LD(wavelength,'Au','D');
eps_m = LD(wavelength,'H2O','D');
lbda = 1/2*(eps_c+eps_m)./(eps_c-eps_m);

option = 2;

if option == 1
    doSingleNanoparticleRadii(wavelength, lbda)
elseif option == 2
    doSingleNanoparticleDistance(wavelength, lbda)
elseif option == 3
    doMultipleNanoparticles(wavelength, lbda)
end
end

% Single nanoparticle disc with various radii
function doSingleNanoparticleRadii(wavelength, lbda)
rMin = 0.1;
rRange = linspace(rMin, rMin*4, 4);
nbPoints = 2^7; %number of points on the boundary

for ir=1:length(rRange);
    r = rRange(ir);
    a = r; b = r; % parameters of the ellipse
    D = ellipse(a,b,nbPoints); % Definition of the ellipse
    D = D + [0;0.5];
    Aire  = D.points(1,:).*D.normal(1,:)*D.sigma';
    Kperiod = KDperiodic(D);
    %Speriod = sort(eig(Kperiod)); % spectrum of Kperiodic
    z = zeros(1,size(lbda,2));
    parfor j=1:size(lbda,2)
        %         fprintf('j: %d\n', j);
        phi = (lbda(j)*eye(D.nbPoints)-Kperiod)\D.normal(2,:)';
        %z(j) = Z(D,phi);
        z(j) = -D.points(2,:).*D.sigma*phi;
    end
    figure(1)
    hold on
    plot(D.points(1,:),D.points(2,:)), hold on, axis equal tight, xlim([-0.5 0.5]), ylim([0 1]);
    figure(2)
    hold on
    plot(wavelength, abs(z), 'LineWidth', 1.5);
    ylabel('|\alpha_{\infty}|');
    xlabel('wavelength')
end

VariableNames = arrayfun(@num2str, rRange, 'UniformOutput', false);
hLeg = legend(VariableNames{:});
set(hLeg, 'FontSize', 12);
end

% Single nanoparticle disc at various distances from the boundary
function doSingleNanoparticleDistance(wavelength, lbda)
dRange = [0.25, 0.75];
nbPoints = 2^7; %number of points on the boundary
a = 0.2; b = 0.2; % parameters of the ellipse

for id=1:length(dRange);
    d = dRange(id);
    D = ellipse(a,b,nbPoints); % Definition of the ellipse
    D = D + [0; d];
    Aire  = D.points(1,:).*D.normal(1,:)*D.sigma';
    Kperiod = KDperiodic(D);
    %Speriod = sort(eig(Kperiod)); % spectrum of Kperiodic
    z = zeros(1,size(lbda,2));
    parfor j=1:size(lbda,2)
        %         fprintf('j: %d\n', j);
        phi = (lbda(j)*eye(D.nbPoints)-Kperiod)\D.normal(2,:)';
        %z(j) = Z(D,phi);
        z(j) = -D.points(2,:).*D.sigma*phi;
    end
    figure(3)
    hold on
    plot(D.points(1,:),D.points(2,:)), hold on, axis equal tight, xlim([-0.5 0.5]), ylim([0 1]);
    figure(4)
    hold on
    plot(wavelength, abs(z), 'LineWidth', 1.5, 'LineSmoothing','on');
    ylabel('|\alpha_{\infty}|');
    xlabel('wavelength')
end

VariableNames = arrayfun(@num2str, dRange, 'UniformOutput', false);
hLeg = legend(VariableNames{:});
set(hLeg, 'FontSize', 12);
end

% Multiple nanoparticle discs
function doMultipleNanoparticles(wavelength, lbda)
nbPoints = 2^7; %number of points on the boundary
a = 0.8; b = 0.8; % parameters of the ellipse
% nbPoints = 2^5; % number of points on the boundary
D = ellipse(a,b,nbPoints); % Definition of the ellipse
Bpoints = []; Btvec = []; Bavec = [];
N = 3; % number of cercles

hFig = figure;
set(hFig, 'Position', [100, 100, 1000, 500]);
parfor j=1:N
    [xt,yt] = pol2cart(2*pi/N*(j-1),1);
    Daux = D*(0.9*sin(pi/N)) + [xt,yt]';
    Bpoints = [Bpoints Daux.points]; Btvec = [Btvec Daux.tvec]; Bavec = [Bavec Daux.avec];
end
B = C2boundary(Bpoints, Btvec, Bavec);
%B = (B < 0)*(0.9/(2+1.8*sin(pi/N)));
Aire2  = B.points(1,:).*B.normal(1,:)*B.sigma';
B = B*0.15 + [0;0.5];
for j = 1:N
    subplot(1, 2, 1);
    hold on
    plot(B.points(1,(j-1)*nbPoints+1:j*nbPoints),B.points(2,(j-1)*nbPoints+1:j*nbPoints),'black'), hold on, axis equal tight, xlim([-0.5 0.5]), ylim([0 1]);
end
Kperiod = KDperiodic(B);
%Speriod = sort(eig(Kperiod)); % spectrum of Kperiodic
z = zeros(1,size(lbda,2));
parfor j=1:size(lbda,2)
    phi = (lbda(j)*eye(B.nbPoints)-Kperiod)\B.normal(2,:)';
    z(j) = -B.points(2,:).*B.sigma*phi;
end

subplot(1, 2, 2);
hold on
plot(wavelength, abs(z));
ylabel('|\alpha_{\infty}|');
xlabel('wavelength')
end