function K = KDperiodic(D)
% K = KDperiodic(D) returns the matrix representing the kernel of Kstar for a 1d periodic arrange of C2boundary D in a 2d space.
% The periodic Green function for this case is given by Gperiodic(x1-y1,x2-y2) = 1/(4pi)
% log(sinh^2(pi|x2-y2|)+sin^2(pi|x1-y1|)) - 1/(4pi)log(sinh^2(pi|x2+y2|)+sin^2(pi|x1-y1|))
% Then KDperdiodic = (grad(Gperiodic) \cdot normalD) * (measure of \partial D)

M = D.nbPoints;
K = zeros(M, M);

for j = 1:M
    Term1 = ( (2*sin(pi*(D.points(1,j)-D.points(1,:))).*cos(pi*(D.points(1,j)-D.points(1,:)))*pi)./(sinh(pi*abs(D.points(2,j)-D.points(2,:))).^2+sin(pi*(D.points(1,j)-D.points(1,:))).^2)...
        - (2*sin(pi*(D.points(1,j)-D.points(1,:))).*cos(pi*(D.points(1,j)-D.points(1,:)))*pi)./(sinh(pi*abs(D.points(2,j)+D.points(2,:))).^2+sin(pi*(D.points(1,j)-D.points(1,:))).^2) ).*D.normal(1,j);
    Term2 = ( (2*sinh(pi*abs(D.points(2,j)-D.points(2,:))).*cosh(pi*abs(D.points(2,j)-D.points(2,:)))*pi.*sign(D.points(2,j)-D.points(2,:)))./(sinh(pi*abs(D.points(2,j)-D.points(2,:))).^2+sin(pi*(D.points(1,j)-D.points(1,:))).^2)...
      - (2*sinh(pi*abs(D.points(2,j)+D.points(2,:))).*cosh(pi*abs(D.points(2,j)+D.points(2,:)))*pi.*sign(D.points(2,j)+D.points(2,:)))./(sinh(pi*abs(D.points(2,j)+D.points(2,:))).^2+sin(pi*(D.points(1,j)-D.points(1,:))).^2) ).*D.normal(2,j);
    
    %off-diagonal terms
    K(j, 1:j-1) = 1/(4*pi)* ...
        (Term1(1:j-1)+Term2(1:j-1)).*D.sigma(1:j-1);
    K(j, j+1:M) = 1/(4*pi)* ...
        (Term1(j+1:M)+Term2(j+1:M)).*D.sigma(j+1:M);
    
    %diagonal term
    K(j,j) = -1/(4*pi)* ...
        D.avec(:,j)'*D.normal(:,j)/(D.tvec_norm(j)^2)*D.sigma(j) ... 
        - 1/(4*pi)*(2*sinh(pi*abs(2*D.points(2,j)))*cosh(pi*abs(2*D.points(2,j)))*pi*sign(D.points(2,j)))/(sinh(2*pi*abs(D.points(2,j))).^2)*D.normal(2,j)*D.sigma(j);
end

end